/* heapsort.c -- Generic routines for sorting an integer array, given a
 *               comparison function.
 *
 * Originally written by Ping?
 * Date -- 10/5/94 -- separated from volbl.c by Mike?
 * Date -- 7/10/95 -- converted (back?) to ANSI C by Ernst.
 */

/*-----------------------------------------------------------------------*/

#include "basic.h"

/* #define HEAP_TEST 1 */

/*-----------------------------------------------------------------------*/

static int heap_size;
static int *heap;
static int (*compare) (int, int);

/*-----------------------------------------------------------------------*/

static void heapify (int i);

/*-----------------------------------------------------------------------*/

#define Parent(i) ((i)/2)
#define Left(i) (2*(i))
#define Right(i) (2*(i)+1)

#define Larger(i, j) (compare (heap[i], heap[j]) == 1)

#define Exchange(i, j) \
do {                    \
   heap[0] = heap[(i)];  \
   heap[(i)] = heap[(j)]; \
   heap[(j)] = heap[0];    \
   } once

/*-----------------------------------------------------------------------*/

void basic_heapsort (int table[], int size, int (*comp) (int, int))
     /* Input/Output: table[1..size]. */
     /* This function will sort table[1..size] using the heapsort algorithm. */
{
  int i, j;
  int temp;

  heap = table;
  heap_size = size;
  compare = comp;

#ifdef HEAP_TEST
  {
    int cnt;

    print ("Heap1 %d:\n", heap_size);
    upfor (cnt, 1, size)
      print ("%d ", table[cnt]);
    print ("\n\n");
  }
#endif

  upfor (i, 2, heap_size)
    {
      temp = heap[i];

#ifdef HEAP_TEST
      {
        int cnt;

        print ("Making a heap %d:\n", i);
        upfor (cnt, 1, size) {
          print ("%d ", table[cnt]);
          if (cnt == (i-1))
            print ("| ");
        }
        print ("\n\n");
      }
#endif

      for (j = i; (j > 1) and (compare (temp, heap[Parent (j)]) > 0);
           j = Parent (j))
        heap[j] = heap[Parent (j)];

      heap[j] = temp;
    }

#ifdef HEAP_TEST
  {
    int cnt;

    print ("Heap2 %d:\n", heap_size);
    upfor (cnt, 1, size)
      print ("%d ", table[cnt]);
    print ("\n\n");
  }
#endif

  while (heap_size > 1)
    {
#ifdef HEAP_TEST
      {
        int cnt;

        print ("Heap of size %d:\n", heap_size);
        upfor (cnt, 1, size) {
          print ("%d ", table[cnt]);
          if (cnt == heap_size)
            print ("| ");
        }
        print ("\n\n");
      }
#endif

      Exchange (1, heap_size);
      heap_size --;
      heapify (1);
    }

  upfor (i, 1, size-1) {
    if (compare (heap[i],heap[i+1]) > 1)
      basic_error ("\nFunction heapsort:  array not sorted!\n");
  }

}

/*--------------------------------------------------------------------------*/

static void heapify (int i)
{
  int largest;
  if (Left (i) <= heap_size)
    {
      if (Left (i) < heap_size)
        largest = If(Larger (Left (i), Right (i)), Left (i), Right (i));
      else
        largest = heap_size;
      if (Larger (largest, i))
        {
          Exchange (largest, i);
          heapify (largest);
        }
    }
}
