/* 
 * delcx/lambda2.c
 * 
 * Author: Mike Facello
 *
 * This file contains a version of lambda2 that accepts column
 * numbers as parameters.  This allows for the computation of the
 * determinant of perturbed submatrices.                                 
 */

/*--------------------------------------------------------------------------*/

#include "basic.h"
#include "sos.h"
#include "../sos/internal.h"
#include "../sos/primitive.h"

/*--------------------------------------------------------------------------*/

Static_Declarations ("lambda2", 2, 14, 2);

#include "../sos/primitive.i.c"

/*--------------------------------------------------------------------------*/

SoS_primitive_result * sos_lambda2_col (int i, int j, int c)
     /* Returns significant term of Lambda2 epsilon-determinant.
        Assumes indices in proper range, pairwise different, and sorted. */
{
#ifdef __SOS_DEBUG__
  {
    lia_clear ();
    print ("sos_lambda2_col (%d,%d) with column %d", i, j, c);
    print (" (");
    print ("%s,1;", Pi(i,c));
    print ("%s,1;", Pi(j,c));
    print (")\n");
  }
#endif
/* modification of code generated by ccode from gee file "lambda3" */
Initialize ();
Epsilon_Term (0);
Positive_Coefficient (Minor2 (i, j, c, 0));
Epsilon_Term (1);
Epsilon (i,c);
Coefficient (Integer (1));
Finish ();
}
