/* 
 * delcx/lambda3.c
 * 
 * Author: Mike Facello
 *
 * This file contains a version of lambda3 that accepts column
 * numbers as parameters.  This allows for the computation of the
 * determinant of perturbed submatrices.                                 
 */

/*--------------------------------------------------------------------------*/

#include "basic.h"
#include "sos.h"
#include "../sos/internal.h"
#include "../sos/primitive.h"

/*--------------------------------------------------------------------------*/

Static_Declarations ("lambda3_col", 3, 14, 2);

#include "../sos/primitive.i.c"

/*--------------------------------------------------------------------------*/

SoS_primitive_result * sos_lambda3_col (int i, int j, int k, int c1, int c2)
     /* Returns significant term of Lambda3 epsilon-determinant.
        Assumes indices in proper range, pairwise different, and sorted.
	Allows the columns used to be chosen.  c1 and c2 are the columns 
	used.                                                              */
{
#ifdef __SOS_DEBUG__
  {
    lia_clear ();
    print ("sos_lambda3_col (%d,%d,%d) with columns %d and %d", i, j, k, 
	   c1, c2);
    print (" (");
    print ("%s,%s,1;", Pi(i,c1), Pi(i,c2));
    print ("%s,%s,1;", Pi(j,c1), Pi(j,c2));
    print ("%s,%s,1;", Pi(k,c1), Pi(k,c2));
    print (")\n");
  }
#endif
/* code generated by ccode from gee file "lambda3" */
Initialize ();
Epsilon_Term (0);
Positive_Coefficient (Minor3 (i, j, k, c1, c2, 0));
Epsilon_Term (1);
Epsilon (i,c2);
Negative_Coefficient (Minor2 (j, k, c1, 0));
Epsilon_Term (2);
Epsilon (i,c1);
Positive_Coefficient (Minor2 (j, k, c2, 0));
Epsilon_Term (3);
Epsilon (j,c2);
Positive_Coefficient (Minor2 (i, k, c1, 0));
Epsilon_Term (4);
Epsilon (j,c2);
Epsilon (i,c1);
Coefficient (Integer (1));
Finish ();
}
