/* delcx/misc.c --- Miscellaneous routines for detri. */
/* author -- Ernst Mucke */
/* modified for delcx by -- Mike Facello */

/*--------------------------------------------------------------------------*/

#include "delcx.h"

/*--------------------------------------------------------------------------*/

static int lexi_comparisons;

/*------------------------------------------------------------------------*/
/* --------------------- Some auxiliary routines ------------------------ */

/*
 * randomize(v, n) -- randomizes the order of the n integers stored in the
 *                    array v.
 */
void randomize(int v[], int n)
{
  int i, j, temp;

  for (i = n; i >= 2; i--) {
    j = (rand() % i) + 1;		/* 1 <= j <= i */
    temp = v[j];
    v[j] = v[i];
    v[i] = temp;
  }
}

/*--------------------------------------------------------------------------*/

/*
 * load_point(i, x, y, z) -- Loads point (x,y,z) into SoS parameter i.
 *			     Only used for infinite points!
 */
void load_point(int i, long x, long y, long z)
{
  Lia lx[3], ly[3], lz[3];

  lia_load (lx, x);   
  lia_load (ly, y);
  lia_load (lz, z);
  sos_param (i, 1, lx);
  sos_param (i, 2, ly);
  sos_param (i, 3, lz);
}

/*--------------------------------------------------------------------------*/

/*
 * scan_vertices () -- This function removes duplicate vertices.  If two
 *                     vertices have the same coordinate, but different
 *                     weights, the largest weighted point is used.
 */

void scan_vertices (int n0, int *n1, int vertex[], int randomized, 
		    int proto_flag)
     /* Dumps "duplicates" and "sorts" vertex[1..*n1] as below:
        randomized == 0 ... sort wrt SOME axis, under SoS;
                   == 1 ... in input order;
                    > 1 ... in random order ==> srandom (randomized) */
{
  int i, j = 1;
  print ("%s %d vertices ...\n",
         If ((randomized == 0), "Sorting", "Scanning"), n0);
  upfor (i, 1, n0)
    vertex[i] = i;
  lexi_comparisons = 0;
  basic_qsort (vertex, 1, n0, lexi_compare);
  upfor (i, 2, n0)
    {
      if (    lia_eq (sos_lia (vertex[i], 1), sos_lia (vertex[j], 1))
          and lia_eq (sos_lia (vertex[i], 2), sos_lia (vertex[j], 2))
          and lia_eq (sos_lia (vertex[i], 3), sos_lia (vertex[j], 3)))
        {
          if (proto_flag)
            print ("     Dumping duplicate vertex %d.\n", vertex[i]-4);
        }
      else
        {
          j ++;
          if (j < i)
            vertex[j] = vertex[i];
        }
    }
  if (j < n0)
    {
      print ("        %d duplicates dumped.\n", n0 - j);
      print ("        %d vertices left.\n", j);
    }
  *n1 = j;
}

/*--------------------------------------------------------------------------*/

/*
 * lexi_compare (i,j) -- Returns -1 if i > j lexicographically.
 *			 Note: The fourth coordinate is the sum of squares
 * 			 MINUS the weight, thus sorting in reverse order
 * 			 by weight.
 */

int lexi_compare (const int *i, const int *j)
{
  lexi_comparisons ++;
  if (lia_eq (sos_lia (*i, 1), sos_lia (*j, 1)))
    {
      if (lia_eq (sos_lia (*i, 2), sos_lia (*j, 2)))
        {
	  if (lia_eq (sos_lia (*i, 3), sos_lia (*j, 3)))
	    {
	      if (lia_eq (sos_lia (*i, 4), sos_lia (*j, 4)))
		return (0);
	      else if (lia_le (sos_lia (*i, 4), sos_lia (*j, 4)))
		return (-1);
	      else
		return (1);
	    }
	  else if (lia_le (sos_lia (*i, 3), sos_lia (*j, 3)))
	    return (-1);
	  else
	    return (1);
        }
      else if (lia_le (sos_lia (*i, 2), sos_lia (*j, 2)))
        return (-1);
      else
        return (1);
    }
  else if (lia_le (sos_lia (*i, 1), sos_lia (*j, 1)))
    return (-1);
  else
    return (1);
}

/*--------------------------------------------------------------------------*/

