/* delcx/regular.c --- 3-dim regular/Delaunay triangulations using flip. */

/* compile with cc -DEBUG for debugging */

/*--------------------------------------------------------------------------*/

#include "delcx.h"
#include "dt.h"

/*--------------------------------------------------------------------------*/

#define TRACE_BLOCK 10
#define PEDANTIC_MOUNT 0
#define PEDANTIC_TEST 0

/*--------------------------------------------------------------------------*/

/* flags and proto macros --- for debugging only */

int regular_test_flag = FALSE;
int regular_proto_flag = FALSE;

#ifdef __DEBUG__
int regular_trace_mode = 2;
#define proto(PRINT_COMMAND)  if (regular_proto_flag) PRINT_COMMAND
#else
int regular_trace_mode = 1;
#define proto(PRINT_COMMAND)  /* do nothing */
#endif

/* regular_trace_mode:  0 ... silent
                        1 ... minimal tracing
                        2 ... normal tracing
                        (assumed when any flag set!)
                        (see also: flip.c) */

/*--------------------------------------------------------------------------*/

static void (* message_hook) (int) = NULL;

static void trace_start(int a, int b, int c, int vau);
static void trace_before(int index, int v);
static void trace_after(int index, int flips);
static void trace_finish(int n);
static void minimal_trace(int n, int f);
static void print_flips(int n, int flips[]);

/*--------------------------------------------------------------------------*/

int search_type = CASCADE_SEARCH;

/*--------------------------------------------------------------------------*/

void compute_triangulation (int vertex[], int n)
{
  int index, visi, c, flips;
  ;
#ifdef __DEBUG__
  if (    (regular_proto_flag or flip_proto_flag)
      and (regular_trace_mode < 2))
    { 
      regular_trace_mode = 2;
      print ("regular: regular_trace_mode is automatically set to 2\n");
    }
#endif
  ;

  flip_list (2 * n);
  search_open (search_type);
  construct_outer_tetra ();
  search_init(vertex, n);

  trace_start (vertex[1], vertex[2], vertex[3], vertex[4]);
  for (index = 1; index <= n-4; index++) {
    trace_before (index, vertex[index]);
    flips = search_insert (vertex[index]);
#ifdef SDEBUG
    printf("\n\nRunning Pedantic Test\n\n");
    Assert_always(dt_test ());
#endif
    trace_after (index, flips);
  }

#ifdef REGULAR
  print_redundant_vertices ();
#endif

  printf ("\n");

}

/*--------------------------------------------------------------------------*/

static void trace_start (int a, int b, int c, int vau)
{
  int index;
#ifdef __DEBUG__
  if (    (regular_proto_flag or flip_proto_flag)
      and (regular_trace_mode < 2))
    { 
      regular_trace_mode = 2;
      print ("regular: regular_trace_mode is automatically set to 2\n");
    }
#endif
  if (regular_trace_mode > 1)
    print ("PROCESSING VERTEX %d AND %d AND %d PLUS %d.\n", a, b, c, vau);
  if (regular_trace_mode == 1)
    upfor (index, 1, 4)
      minimal_trace (index, 0);    
}

static void trace_before (int index, int v)
{
  switch (regular_trace_mode)
    {
     case 0:
     case 1:
      break;
     default:
      print ("ADDING VERTEX[%d] %d\n", index, v);
    }
}

static void trace_after (int index, int flips)
{
  if (regular_trace_mode == 1)
    minimal_trace (index, flips);
  if (message_hook)
    message_hook (index);
}

static void trace_finish (int n)
{
  if (regular_trace_mode == 1)
    minimal_trace (n, -1);  /* flush */
}  

static void  minimal_trace (int n, int f)
{
  static int flips[TRACE_BLOCK];
  int m = n mod TRACE_BLOCK;
  int j;
  if (f < 0)
    { /* flush */
      print_flips (n, flips);
      return;
    }
  if (n == 1)
    upfor (j, 0, TRACE_BLOCK - 1)
      flips[j] = -1;  /* clear */
  flips [If (m, m - 1, TRACE_BLOCK - 1)] = f;
  if (m == 0)
    {
      print_flips (n, flips);
      upfor (j, 0, TRACE_BLOCK - 1)
        flips[j] = -1;  /* clear */
    }
}

static void print_flips (int n, int flips[])
{
  int j;
  if (flips[0] < 0)
    return; /* there is nothing to print */
  print ("v[%d]...", ((n - 1) / TRACE_BLOCK) * TRACE_BLOCK + 1);
  upfor (j, 0, TRACE_BLOCK - 1)
    if (flips[j] >= 0)
      print (" %d", flips[j]); 
  print ("\n"); 
}
/*--------------------------------------------------------------------------*/
