/* sos/rho1.c */

/*--------------------------------------------------------------------------*/

#include "basic.h"
#include "sos.h"
#include "../sos/internal.h"

/*--------------------------------------------------------------------------*/

void sos_rho1 (int i, int j, Lia_obj a, Lia_obj b)
     /* Output: a, b. */
     /* Calculates Rho1 == a/b (the radius of the circumsphere of) the
        edge in R^3 given by its endpoints: indices i, j.
        Indices in proper range and pairwise different.
        NOTE: Either a != 0 and b > 0
              or a/b == 0/0 (1/0) for special values zero (infinity). */
{
  SoS_primitive_result *num;
#ifdef __DEBUG__
  if (sos_proto_flag)
    print ("sos_rho1 (%d,%d,%d)\n", i, j);
#endif
  ;
  /* compute Rho1 w/ SoS... saving the Lia pointer immediately! */
  (void) basic_isort2 (&i, &j);
  num = sos_rho1_num (i, j);
  lia_copy (num->lia_pointer, a);
  lia_copy (lia_const (4), b);  /* denumerator is constant! */
  ;
  /* map from SoS to a ratio... */
  if (num->depth == 0)
    { /* eps(num) == eps(den) == 1  ==>  normal ratio: a/b */
      ; /* no correction necessary*/
    }
  else 
    { /* eps(num) < eps(den) == 1  ==>  zero: 0/0 */
      lia_copy (lia_const (0), a);
      lia_copy (lia_const (0), b);
      print ("<Rho1 zero>\n");
    }
  ;
#ifdef __DEBUG__
  if (sos_test_flag)
    if (not ((0 <= i) and (i < j) and (j <= sos_common.n)))
      basic_error ("sos_rho1: arguments were wrong");
  if (sos_proto_flag)
    {
      lia_clear ();
      print ("sos_rho1 (%d,%d) result: %s/%s (%d)\n",
             i, j, lia_deci (a), lia_deci (b), num->depth);
    }
#endif
}
