/* sos/rho3.c */

/*--------------------------------------------------------------------------*/

#include "basic.h"
#include "sos.h"
#include "../sos/internal.h"

/*--------------------------------------------------------------------------*/

void sos_rho3 (int i, int j, int k, int l, Lia_obj a, Lia_obj b)
     /* Output: a, b. */
     /* Calculates Rho3 == a/b (the radius of the circumsphere of) the 
        tetrahedron given by its spanning vertices: indices i, j, k, l.
        Indices in proper range and pairwise different.
        NOTE: Either a != 0 and b > 0
              or a/b == 0/0 (1/0) for special values zero (infinity). */
{
  SoS_primitive_result *num, *den;
#ifdef __DEBUG__
  if (sos_proto_flag)
    print ("sos_rho3 (%d,%d,%d,%d)\n", i, j, k, l);
#endif
  ;
  /* compute ratio w/ SoS... saving the Lia pointer immediately! */
  (void) basic_isort4 (&i, &j, &k, &l);
  num = sos_rho3_num (i, j, k, l);
  lia_copy (num->lia_pointer, a);
  den = sos_rho3_den (i, j, k, l);
  lia_copy (den->lia_pointer, b);
  ;
  /* map from SoS to a ratio... */
  if ((num->depth == den->depth) and (num->depth == 0))
    { /* eps(num) == eps (den) == 1  ==>  normal ratio: a/b */
      ; /* no correction necessary*/
    }
  else
    {
      switch (sos_epsilon_compare (num, den))
        {
         case 1:
          { /* eps(num) > eps(den)  ==>  infinity: 1/0 */
            lia_copy (lia_const (1), a);
            lia_copy (lia_const (0), b);
            break;
          }
          /* NOTE: Cases -1 and 0 don't seem to happen. !?! */
         case -1:
          { /* eps(num) < eps(den)  ==>  zero: 0/0 */
            lia_copy (lia_const (0), a);
            lia_copy (lia_const (0), b);
            print ("<Rho3 zero>\n");
            break;
          }
         default:
          { /* eps(num) == eps(den) != 1  ==> degenerated ratio: a/b */
            ; /* leave it as is */
            print ("<Rho3 degenerated a/b [%d/%d]>\n", num->depth, den->depth);
            break;
          }
        }
    }
  ;
#ifdef __DEBUG__
  if (sos_test_flag)
    if (not ((0 <= i) and (i < j) and (j < k) and (k < l)
             and (l <= sos_common.n)))
      basic_error ("sos_rho3: arguments were wrong");
  if (sos_proto_flag)
    {
      lia_clear ();
      print ("sos_rho3 (%d,%d,%d,%d) result: %s/%s (%d,%d;%d)\n",
             i, j, k, l, lia_deci (a), lia_deci (b),
             num->depth, den->depth, sos_epsilon_compare (num, den) );
    }
#endif
}
